<?php
namespace ReynaertBe\DIP;
class Service {
    private $debug = false;
    private $url = null;

    /**
     * Method: POST, PUT, GET etc
     * Data: array("param" => "value") ==> index.php?param=value
     * $transform is een functie die het resultaat gaat omvormen; bv 'HTML::safeInt' (als een string)
     */
    //
    public function CallAPI($method, $url, $data = false, $transform = null) {
        $url = $this->url.$url;
        $curl = curl_init();

        switch ($method)
        {
            case DIP::METHOD_POST:
                curl_setopt($curl, CURLOPT_POST, 1);
                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            case DIP::METHOD_PUT:
                curl_setopt($curl, CURLOPT_PUT, 1);
                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            case DIP::METHOD_DELETE:
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
                if ($data)
                    $url = sprintf("%s?%s", $url, http_build_query($data));
                break;
            case DIP::METHOD_PATCH:
                curl_setopt($curl, CURLOPT_POST, 1);
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PATCH");
                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            default:
                if ($data)
                    $url = sprintf("%s?%s", $url, http_build_query((is_string($data))?array($data):$data));
        }

        $headers = [];
        $headers['Content-Type'] = 'application/json';

        // Authentication:
        $bearer = $this->getBearer();
        $headers['Authorization'] = 'Bearer ' . $bearer;
        curl_setopt($curl, CURLOPT_HTTPHEADER, $this->formatHeader($headers));

        if ($this->debug) {
            echo "<pre>URL: ".$url."</pre>";
            echo "<pre>Data: ".print_r($data, true)."</pre>";
            curl_setopt($curl, CURLOPT_VERBOSE, true);
        }

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

        // poging 1
        $result = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($httpCode == '401') { // unauthorised
            $bearer = $this->getBearer(true);
            $headers['Authorization'] = 'Bearer ' . $bearer;
            curl_setopt($curl, CURLOPT_HTTPHEADER, $this->formatHeader($headers));
            // poging 2
            $result = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            if ($this->debug) {
                echo "<pre>Authenticatie HttpCode: ".$httpCode."</pre>";
                echo "<pre>Result: ".print_r($result, true)."</pre>";
            }
        }

        if($httpCode>=400 && $httpCode<=599){
            $result_controle = json_decode($result);
            if(is_null($result_controle) && is_string($result) && $result!="") $result = json_encode(array(json_decode(json_encode(array('_error' => $httpCode, '_major' => true, 'message' => "Error: ".$result), JSON_FORCE_OBJECT))));
            elseif(is_object($result_controle) && isset($result_controle->title)) $result = json_encode(array(json_decode(json_encode(array('_error' => $httpCode, '_major' => true, 'message' => "Error: ".$result_controle->title), JSON_FORCE_OBJECT))));
            else $result = json_encode(array(json_decode(json_encode(array('_error' => $httpCode, '_major' => true, 'message' => "Er is een (niet gespecificeerde ) fout opgetreden."), JSON_FORCE_OBJECT))));
        } else if ($httpCode == 200)
            $this->updateBearer($result);

        if ($this->debug) {
            echo $result."</br>";
            printf("cUrl error (#%d): %s<br>\n", curl_errno($curl),
                htmlspecialchars(curl_error($curl)));
        }

        curl_close($curl);

        if (!is_null($transform)) $result = $transform($result);

        return $result;
    }

    private function formatHeader($assoc) : array {
        $headers = [];
        foreach ($assoc as $key => $value) {
            $headers[] = $key.': '.$value;
        }
        return $headers;
    }

    public function debug($on=true) : self {
        $this->debug = $on;
        return $this;
    }

    private function getBearer($refresh = false) {
        $bearer = $_SESSION['factureBearer']??'';
        if (is_null($bearer) || strlen($bearer)==0 || $refresh) {
            $ch = curl_init(DIP::getResourceToken());
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            $post = [
                'email' => DIP::getEmail(),
                'password' => DIP::getPassword()
            ];
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
            $response = curl_exec($ch);
            $r = json_decode($response, true);
            if (isset($r['error'])) {
                throw new \Exception('The authentication with defactu.re web application failed. Please check the credentials');
            }
            $_SESSION['factureBearer'] = $bearer = $r['token'];
        }
        return $bearer;
    }

    private function updateBearer(string $result) : self  {
        $data = json_decode($result, true);
        if (isset($data["token"])) $_SESSION['factureBearer'] = $data["token"];
        return $this;
    }

    public function __construct() {
        $this->url = DIP::getResourceBase();
        return $this;
    }
}