<?php
namespace ReynaertBe\DeFactuRe;

class WCSettingsPage extends \WC_Settings_Page {

    public function __construct() {

        $this->id = 'settings-reynaert-defacture';
        $this->label = __( 'DeFactu.re', 'reynaert-facture' );
        parent::__construct();
        

        /**
         *	Define all hooks instead of inheriting from parent
         */

        // Add the tab to the tabs array
        add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 99 );

        // Add new section to the page
        add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );

        // Add settings
        add_action( 'woocommerce_settings_' . $this->id, array( $this, 'output' ) );

        // Process/save the settings
        add_action( 'woocommerce_settings_save_' . $this->id, array( $this, 'save' ) );
    }

    /**
     *	Get sections
     *
     *	@return array
     */
    public function get_sections() {

        // Must contain more than one section to display the links
        // Make first element's key empty ('')
        $sections = array(
            '' => __( 'Settings', 'reynaert-facture' ),
            'readme' => __( 'Readme', 'reynaert-facture' )
        );

        return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );
    }

    /**
     *	Output sections
     */
    public function output_sections() {

        global $current_section;

        $sections = $this->get_sections();

        if ( empty( $sections ) || 1 === sizeof( $sections ) ) {
            return;
        }

        echo '<ul class="subsubsub">';

        $array_keys = array_keys( $sections );

        foreach ( $sections as $id => $label ) {
            echo '<li><a href="' . admin_url( 'admin.php?page=wc-settings&tab=' . $this->id . '&section=' . sanitize_title( $id ) ) . '" class="' . ( $current_section == $id ? 'current' : '' ) . '">' . $label . '</a> ' . ( end( $array_keys ) == $id ? '' : '|' ) . ' </li>';
        }

        echo '</ul><br class="clear" />';
    }

    /**
     *	Get settings array
     *
     *	@return array
     */
    public function get_settings() {

        global $current_section;

        $settings = array();

        if ( $current_section == '' ) {

            $settings = [

                /**
                 *	For settings types, see:
                 *	https://github.com/woocommerce/woocommerce/blob/fb8d959c587ee95f543e682e065192553b3cc7ec/includes/admin/class-wc-admin-settings.php#L246
                 */

                // Settings input
                [
                    'title' => __( 'API Settings', 'reynaert-facture' ),
                    'type' => 'title',
                    'desc' =>  __( 'Manage your settings for the WooCommerce - Reynaert - defacture plugin.', 'reynaert-facture' ),
                    'id' => 'woocommerce_reynaert_defacture_settings'
                ],
                [
                    'title' => __( 'Url', 'reynaert-facture' ),
                    'type' => 'text',
                    'desc' => __( 'Add the url off the invoice web application, e.g. https://defactu.re', 'reynaert-facture' ),
                    'desc_tip' => true,
                    'id' => 'woocommerce_reynaert_defacture_url',
                    'css' => 'min-width:300px;',
                ],
                [
                    'title' => __( 'Username', 'reynaert-facture' ),
                    'type' => 'text',
                    'desc' => __( 'The API username', 'reynaert-facture' ),
                    'desc_tip' => true,
                    'id' => 'woocommerce_reynaert_defacture_username',
                    'css' => 'min-width:300px;',
                ],
                [
                    'title' => __( 'Password', 'reynaert-facture' ),
                    'type' => 'password',
                    'desc' => __( 'The API password', 'reynaert-facture' ),
                    'desc_tip' => true,
                    'id' => 'woocommerce_reynaert_defacture_password',
                    'css' => 'min-width:300px;',
                ],
                [
                    'type' => 'sectionend',
                    'id' => 'woocommerce_reynaert_defacture_sectionend'
                ],
            ];

        } else {

            // Overview
            $settings = [
                [
                    'title' => __( 'Application url', 'reynaert-facture' ),
                    'type' => 'title',
                    'desc' =>  __( 'Browse to your facture webapplication and copy the url. Now paste this to the url field, but leave the subdirectory away.<br>E.g. https://defactu.re/admin/ becomes https://defactu.re', 'reynaert-facture' ),
                    'id' => 'woocommerce_reynaert_defacture_settings'
                ],
                [
                'title' => __( 'API user', 'reynaert-facture' ),
                'type' => 'title',
                'desc' =>  __( 'Best practice is to make a specific API user. If you don\'t have this rights contact support@reynaert.be', 'reynaert-facture' ),
                'id' => 'woocommerce_reynaert_defacture_settings'
            ]
            ];
        }

        return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings );
    }

    /**
     *	Output the settings
     */
    public function output() {
        $settings = $this->get_settings();
        \WC_Admin_Settings::output_fields( $settings );
    }

    /**
     *	Process save
     *
     *	@return array
     */
    public function save() {

        global $current_section;

        $settings = $this->get_settings();

        \WC_Admin_Settings::save_fields( $settings );

        if ( $current_section ) {
            do_action( 'woocommerce_update_options_' . $this->id . '_' . $current_section );
        }
    }
}