<?php
/*
 * Copyright (c) 2022 Reynaert.be -- all rights reserved
 *
 * Reynaert.be BVBA. ("Reynaert.be") permits use of the
 * accompanying software and documentation ("the Software") provided
 * that the conditions of the Reynaert.be License
 * ("the License") are met. 
 *
 * This Software is provided "AS IS", without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED REPRESENTATIONS AND WARRANTIES, INCLUDING 
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED.
 *
 * REYNAERT.BE AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE OR ANY THIRD PARTY AS A RESULT OF USING OR
 * DISTRIBUTING SOFTWARE. IN NO EVENT WILL REYNAERT.BE OR ITS LICENSORS 
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, 
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, 
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF HE HAS 
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
/**
 * @package Reynaert.be
 * @author    Johan Reynaert <johan@reynaert.be>
 * @copyright 2022 Reynaert.be
 * @license   MIT http://opensource.org/licenses/MIT
 * @version   1.0.0
 * @link      https://www.reynaert.be/
 */
/*
Plugin Name: WooCommerce defactu.re
Plugin URI: https://www.reynaert.be/
Description: Make an invoice with the https://defactu.re/ webapplication
Version: 1.0.0
Author: Johan Reynaert
Author URI: https://www.reynaert.be/wie-wat/johan-reynaert/
*/


// Make sure we don't expose any info if called directly
if ( !function_exists( 'add_action' ) ) {
	echo 'Hi there!  I\'m just a plugin, not much I can do when called directly.';
	exit;
}

/* If a version of code snippets has already been loaded, then deactivate this plugin. */
if ( defined( 'REYNAERT_BE_DEFACTURE_FILE' ) ) {
	require_once ABSPATH . 'wp-admin/includes/plugin.php';
	deactivate_plugins( array( 'reynaert-facture/reynaert-facture.php' ), true );

	if ( ! function_exists( 'reynaert_be_defacture_deactivated_old_version_notice' ) ) {
		function reynaert_be_defacture_deactivated_old_version_notice() {
			echo '<div class="error fade"><p>',
			esc_html__( 'Another version of Reynaert.be Defactu.re appears to be installed. Deactivating this version.', 'reynaert_be_defacture' ),
			'</p></div>';
		}
	}

	add_action( 'admin_notices', 'reynaert_be_defacture_deactivated_old_version_notice', 11 );
	return;
}

define( 'REYNAERT_BE_DEFACTURE_VERSION', '1.0.0' );
define( 'REYNAERT_BE_DEFACTURE_FILE', __FILE__ );
define( 'REYNAERT_BE_DEFACTURE_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );

defined ('ERROR_NOTIFY_EMAIL') OR define('ERROR_NOTIFY_EMAIL','johan@reynaert.be'); //Notify on error ...
defined ('SUPPORT_NOTIFY_EMAIL') OR define('SUPPORT_NOTIFY_EMAIL','johan@reynaert.be');

/**
 * Enable autoloading of namespaced plugin classes
 * @param $class
 */
try {
	spl_autoload_register( function ( $class ) {
		$file = str_replace('\\', DIRECTORY_SEPARATOR, $class).'.php';
	    if (file_exists(REYNAERT_BE_DEFACTURE_PLUGIN_DIR.'classes'.DIRECTORY_SEPARATOR.$file))
	        require_once REYNAERT_BE_DEFACTURE_PLUGIN_DIR.'classes'.DIRECTORY_SEPARATOR.$file;
	});
} catch ( Exception $e ) {
	new WP_Error( $e->getCode(), $e->getMessage() );
}


/** At the end; something like this is what we want */
/*
$invoice = new \ReynaertBe\DeFactuRe\SendInvoice([
    "customer" => [
        "name" => "BVBA Reynaert.be",
        "attn" => "Johan Reynaert",
        "email" => "johan@reynaert.be",
        "address" => "Brandstraat 60",
        "postalcode" => "8755",
        "city" => "Ruiselede",
        "countryCode" => "BE",
        "enterpriseNumber" => "BE0479024206"
    ],
    "title" => "",
    "date" => "2000-12-25",
    "period" => "December 2000",
    "ref" => "A referende",
    "datePaid" => "2000-12-30",
    "paid" => true,
    "department" => 1,
    "lines" => [
        [
            "title" => "Test item 1",
            "quantity" => 1,
            "unitPrice" => 45.02,
            "vat" => 21
        ],
        [
            "title" => "Test item 2",
            "quantity" => 2,
            "unitPrice" => 44.99,
            "vat" => 21
        ]
    ]
]);
echo $invoice->getData('downloadPDF');
echo '<br>'.$invoice->getData('number');
*/

//require_once( REYNAERT_BE_DEFACTURE_PLUGIN_DIR . 'functions.php' );


/** activate hooks */

// bij status completed en klant heeft gekozen voor factuur, maak er dan één
add_action( 'woocommerce_order_status_completed', function ($order_id) {
	if ((int)get_post_meta( $order_id, 'send_invoice', true )==1 && strlen(get_post_meta( $order_id, "invoice_number", true ))===0) {
		$order = wc_get_order( $order_id );
		$data = [
			"customer" => [
		        "name" => $order->get_billing_company(),
		        "attn" => trim($order->get_billing_first_name().' '.$order->get_billing_last_name()),
		        "email" => $order->get_billing_email(),
		        "address" => trim($order->get_billing_address_1().' '.$order->get_billing_address_2()),
		        "postalcode" => $order->get_billing_postcode(),
		        "city" => $order->get_billing_city(),
		        "countryCode" =>  $order->get_billing_country(),
		        "enterpriseNumber" => get_post_meta( $order->get_id(), 'vat_number', true )
		    ],
		    "title" => "[#Shop-".$order_id."] ".get_bloginfo( 'name' ),
		    "date" => date("Y-m-d"),
		    "period" => "",
		    "ref" => "",
		    "datePaid" => date("Y-m-d"),
		    "paid" => true,
		    "department" => 0,
		    "lines" => []
		];
		$_tax = new WC_Tax();
	    foreach ($order->get_items() as $item ) {
	        $product = $item->get_product();
	        $data['lines'][] = [
	        	"title" => $product->get_name(),
	            "quantity" => $item->get_quantity(),
	            "unitPrice" => $product->get_price(),
	            "vat" => $_tax->get_rates($product->get_tax_class())[1]["rate"] // of misschien beter array_shift(array_shift($_tax->get_rates($product->get_tax_class())))
	        ];

	    }
	    if ($order->get_shipping_total() > 0)
	    	$data['lines'][] = [
	        	"title" => "shipping",
	            "quantity" => 1,
	            "unitPrice" => $order->get_shipping_total() - $order->get_shipping_tax(),
	            "vat" =>  round(($order->get_shipping_tax()/$order->get_shipping_total()) * 100)
	        ];

		$invoice = new \ReynaertBe\DeFactuRe\SendInvoice($data);
		update_post_meta($order_id, 'invoice_link', $invoice->getData('downloadPDF'));
		update_post_meta($order_id, 'invoice_number', $invoice->getData('number'));
	}
});
// voeg twee velden toe aan checkout
add_action( 'woocommerce_after_checkout_billing_form', function () {
	woocommerce_form_field( 'checkout_invoice', [
       'type'          => 'checkbox',
       'class'         => array('form-row mycheckbox'), // CSS Class
       'label_class'   => array('woocommerce-form__label woocommerce-form__label-for-checkbox checkbox'),
       'input_class'   => array('woocommerce-form__input woocommerce-form__input-checkbox input-checkbox'),
       'required'      => false,
       'label'         => __('Send me an invoice', 'reynaert-facture'),
    ]);
	woocommerce_form_field( 'checkout_invoice_vat', [
       'type'          => 'text',
       'class'         => array('form-row'), // CSS Class
       'label_class'   => array('woocommerce-form__label'),
       'input_class'   => array('woocommerce-Input woocommerce-Input--text input-text'),
       'required'      => false,
       'label'         => __('Vat number','reynaert-facture'),
    ]);
});
// stuff extra fields into order as meta data
add_action('woocommerce_checkout_update_order_meta', function ($order_id) {
	if (!empty($_POST['checkout_invoice'])) {
		update_post_meta($order_id, 'send_invoice',1);
		update_post_meta($order_id, 'vat_number',sanitize_text_field($_POST['checkout_invoice_vat']));
	}
});
add_action('woocommerce_email_after_order_table', function($order, $sent_to_admin, $plain_text, $email) {
    /*if ( $email->id == 'cancelled_order' ) {}
    if ( $email->id == 'customer_completed_order' ) {}
    if ( $email->id == 'customer_invoice' ) {}
    if ( $email->id == 'customer_new_account' ) {}
    if ( $email->id == 'customer_note' ) {}
    if ( $email->id == 'customer_on_hold_order' ) {}
    if ( $email->id == 'customer_refunded_order' ) {}
    if ( $email->id == 'customer_reset_password' ) {}
    if ( $email->id == 'failed_order' ) {}
    if ( $email->id == 'new_order' ) {}*/
    $iv_number = get_post_meta( $order->get_id(), "invoice_number", true );
    if ( $email->id == 'customer_invoice' || $email->id == 'customer_completed_order')
        if ((int)get_post_meta( $order->get_id(), 'send_invoice', true )===1 && strlen($iv_number)!==0)
            echo sprintf("<p>".__("The invoice with number %s can be <a href=\"%s\">downloaded as PDF</a>",'reynaert-facture'), $iv_number, get_post_meta( $order->get_id(), "invoice_link", true ));
}, 20, 4);

// add some translations
add_action( 'init', function() {
    load_plugin_textdomain( 'reynaert-facture', false, dirname( plugin_basename( REYNAERT_BE_DEFACTURE_FILE ) ). '/locale/' );
});

if(!session_id()) session_start();
// ADMIN
if (is_admin()) {
    add_filter( 'woocommerce_get_settings_pages', function ( $settings ) {
        return new \ReynaertBe\DeFactuRe\WCSettingsPage();
    }, 15);
}
